; ****************************************************************************
;			 FULL ZX80 CHESS IN 1K  -  PLUS
; ****************************************************************************
;
;     Based on an article from the February 1983 issue of 'Your Computer'
;
;     - by David R. Horne
;
;     - modifications by Zsolt Gal (GZS) 2014
;
; ****************************************************************************
;
#define QPawn			; choose this option for "Queen's pawn game"

	; system variables

	.org $4000	;
	;
	; ------------------------------------------------------------ 12 bytes
PSC
	.db $FF		; ERR_NR - error code
	.db $88		; FLAGS
	.dw $FFFE	; PPC	 - line number which is to be executed next
	.dw eof_Vars+1	; E_ADDR
	.dw $0001	; E_PPC	 - line number for LIST
	.dw eof_Prg	; VARS	 - pointer to VARS / end of BASIC PROGRAM
	.dw eof_Vars	; E_LINE - pointer to INPUT BUFFER (-save area end-)
	;
	; ---------------------------------------------------------------------
	;
	.dw eof_Vars+2	; D_FILE - pointer to VRAM / end of INPUT BUFFER
	;
	; ------------------------------------------------------------ 16 bytes
ALIST
	.dw eof_Vars+3	; DF_EA
	.dw eof_Vars+3	; DF_END
	.db $02		; DF_SZ
	.dw $0001	; S_TOP
	.dw $0000	; X_PTR
FillPawn
	.dw $0000	; OLDPPC - line number for CONTINUE
FillOut
	.db $00		; FLAGX
	.dw $07B0	; T_ADDR
	.dw $0000	; SEED	 - random generator seed
	;
	; ---------------------------------------------------------------------
	;
	.dw $241E	; FRAMES - frames counter

	.dw $0000	; V_ADDR
	;
	.dw $FFFF	; ACC (RESULT)

	.dw $1721	; S_POSN

	.dw eof_Vars+1	; CH_ADD - BASIC program pointer
;
;	BASIC PROGRAM @ $4028 (16424) =========================================
;
TL	.equ $+14
FillBrd
	.db $00,$01			; 1. sor
	.db $EF,$3A,$38,$37		; RANDOMISE USR
	.db $DA,$1D,$22,$20,$20,$1D,$D9	; (16441) : $4039 => TKP1
	.db $76				; N/L
	;
	; ---------------------------------------------------------------------
	;
	.db $76,$02,$FE		; Line 2 REM
	;
;
; ************************************************************************
;	MAIN PROGRAM ENTRY (RAND USR 16441)
; ************************************************************************
;
	ld a,8			; we move 8x8 (64) bytes stored
	ld b,$00		; in the chessboard area
	jp START		; 
;
;	------------------------------------------------------------------
;
;	The following (commented ;;) parts of the code are stored in D-File
;	as fields of the chessboard. These (8x8=) 64 bytes overwrite the 
;	area of the system variables ($4000-$403f) during startup.
;
;;	==================================================================
;;
;; PSC: gives a score to a chess piece - Q(5), R(4), B(3), N(2), P(1), K(0)
;;
;;PSC
;;	and $7f			; deleting the colour of the piece
;;	ld hl,whPieces		; the character codes of the pieces
;;	ld bc,$0006		; the highest value +1
;;	cpir                    ; search
;;	ld a,c			; Z=0 if there's no match 
;;	ret			; A=C
;;
;;	==================================================================
;;
;;	.dw DFile		; D_FILE - pointer to VRAM
;;
;;	==================================================================
;;
;; AddList: adds to the current legal move list another entry on the end.
;;
;;ALIST
;;	ld hl,newList		; the starting address of the list of moves
;;	inc (hl)		; increase the counter
;;	ld a,(hl)		; read the counter
;;	add a,l			; calculate the new address
;;	ld l,a			; set pointer
;;	ld (hl),c		; and store the new item
;;	ret	  		;
;;
;;	------------------------------------------------------------------
;;
;;FillPawn
;;	ld b,8			; there are 8 pawns
;;FillOut
;;	dec hl			; set pointer 
;;	ld (hl),c		; fill one 
;;	djnz FillOut		; was it the last?
;;	ret			; yes, return
;;
;;	==================================================================
;;
;;	.dw $0000	; FRAMES - frames counter
;;	.dw $0000	; V_ADDR
;;	.dw $0000	; ACC (RESULT)
;;	.dw $0000	; S_POSN
;;	.dw $0000	; CH_ADD - BASIC program pointer
;;
;;	==================================================================
;;FillBrd
;;	ld hl,blkRook		; the table of the
;;	ld bc,$0808		; 8 pieces
;;FillBrd1
;;	call CHG		; change the colour,
;;	ldi			; then place it
;;	djnz FillBrd1		; was it the last?
;;	ret			; yes, return
;;
;;	------------------------------------------------------------------
;;
;; TestList: tests to see if there are any moves in the move list.
;;
;;TL
;;	ld hl,newList		; starting address of the list of moves
;;	dec (hl)		; lower the counter
;;	ld a,(hl)		; read the counter
;;	inc a			; if the original value
;;	ret z			; is zero, then back
;;	add a,l			; otherwise set the pointer
;;	ld l,a			; to the last item
;;	ld a,(hl)		; return with the value of the last item
;;
;;	==================================================================
;
	ret			; 
;
;	---------------------------- TABLES:
theKing
aRook
	.db $01,$0B,$FF,$F5
	;     1, 11, -1,-11
blkPawn	.equ $+1
aBishop
	.db $F6,$F4,$0C,$0A
	;   -10,-12, 12, 10
akNight
	.db $0D,$F3,$15,$EB,$17,$E9,$F7,$09
	;    13,-13, 21,-21, 23,-23, -9,  9
	.db $0B,$0A,$0C
	;    11, 10, 12
whtPawn	.equ $-1
;
;	------------------------------------------------------------------
;
; The subroutine TKP just scans the keyboard waiting for an appropriate key to
; be depressed. The alpha-numeric entry is then translated to a board address.
;
TKP
	ld (hl),$8F		; the cursor (inverse "?") character
	push hl			; save input line position
	push bc			; save possible character codes
;
;	------------------------------------------------------------------
TKP0
	CALL $013F		; routine KEYBOARD gets key values in BC.

	SRA B			; sets carry if unshifted (bit 7 remains set)
	jr nc,TKP0		; otherwise read again

	LD L,$05		; there are five keys in each row.

TKP1				;; KEY-LINE
	ADD A,L			; add value 5 (or 1)
	SCF			; carry will go to bit 7
	RR C			; test C (which has 1 unset bit identifying row)
	JR C,TKP1		; back if carry to KEY-LINE

	; if only one key pressed C should now be $FF.

	INC C			; test for $FF
	JR NZ,TKP0		; back if multiple keys to TKP0

	; the high byte of the key value identifies the column - again only one bit is
	; now reset.

	LD C,B			; transfer to B
	DEC L			; test if this is first time through
;
;	------------------------------------------------------------------
;
	LD L,$01		; reduce increment from five to one.
	JR NZ,TKP1		; back if L was five to KEY-LINE

	; The accumulator now holds a key value 1-78 decimal.

	LD HL,$006C - 7		; location before the MAIN-KEYS table ($006B)
				; the index value is 1 - 78.
	LD E,A			; code to E  (D is zero from keyboard)
	ADD HL,DE		; index into the table.
;
;	------------------------------------------------------------------
;
	ld a,(hl)		; A= the decoded character
	pop bc			; set back the possible character 
	push bc			; codes for the test
TKP2
	cp c			; if it corresponds,
	jr z,TKP3		; return

	inc c			; next value
	djnz TKP2		; if it's not among the 8 possible value,
 
	jr TKP0			; then start from the beginning
TKP3
	pop bc			; possible character codes
	pop hl			; input position (INPUT)
	ld (hl),a		; display the character
	ret     
;
;	------------------------------------------------------------------
;
; KYBD is a routine which sets up machine control of the keyboard such that only
; the eight key codes from code 29 and the eight key codes from code 38 are
; acceptable entries. Any other key depression is ignored.
;
KYBD
	ld bc,$0926		; possible character codes ("A".."I")
KYBD0
	call TKP		; reading the keyboard 

	inc hl			; next input position
	ld c,$1d		; possible character codes ("1".."9")
	call TKP		; reading the keyboard

	sub $1c			; number from character ("1" -> 1; "8" -> 8)
	ld b,a			; Y (rows)
	xor a			;
KYBD1
	add a,$0b		; 11*Y
	djnz KYBD1		;

	add a,CountA1 & 255	; ($61 originally)
	dec hl			; next input position (column)
	sub (hl)		; A= the screen position's lower byte 
;
;	------------------------------------------------------------------
;
; STR: this routine takes the board address and determines whether the contents
; are: different from the current mover colour (0); empty (1); the board
; surround (2) or the same colour as the current mover (3).
;
STR
	ld c,a			; save the screen position's lower byte
	ld l,c			; L= the screen position's lower byte
	ld h,BoardH1/256	; HL= the whole screen address
;
;	------------------------------------------------------------------
STR2
	ld a,l			; A= the screen position's lower byte
	ld b,$02		; B=2 indicates the margin of the board

	cp BoardH1 & 255	; if the address is smaller than the board's
	jr c,STR4		; first square, see the next possible move

	cp BrdOut & 255		; if the address is bigger than the board's
	jr nc,STR4		; last square, then see the next possible move

	ld a,(hl)		; the board position's content
	and $7f			; deleting colour bit
	jr z,STR3		; empty square?

	cp $76			; right margin (N/L)?
	jr z,STR4		;

	cp $26			; left margin ?
	jr c,STR4		; (character code smaller than "A")

	ld a,(hl)		; colour bit back again
	inc b			; B=3 indicates own piece

MoverCol .equ $+1

	add a,$80		; the current mover's colour ($80 or $00)
	rla			; if it corresponds, the 7th bit
	jr nc,STR4		; is zero

        xor a                   ; set Z-flag
        ret                     ; A=0 indicates the opponent
STR3
        dec b                   ; B=1 indicates the empty square
STR4
	ld a,b			; A=B (1..3) position's properties
        and a                   ; reset Z-flag
	ret			;
;
;	------------------------------------------------------------------
GetPos
	ld hl,INPUT+5		; end of the INPUT LINE (+1)
	call FillOut		; fill the INPUT line
GetPos2
	call KYBD		; get and check the "from" position

	cp $02			; out of board (I1..I9)?
	ret
;
;	---------------------------------------------------------- PIECES:
;
whK	.equ $30		; white King
whQ	.equ $36		; white Queen
whR	.equ $37		; white Rook
whB	.equ $27		; white Bishop
whN	.equ $33		; white kNight
whP	.equ $35		; white Pawn
;
;	---------------------------------
;
blK	.equ whK+$80		; black King
blQ	.equ whQ+$80		; black Queen
blR	.equ whR+$80		; black Rook
blB	.equ whB+$80		; black Bishop
blN	.equ whN+$80		; black kNight
blP	.equ whP+$80		; black Pawn
;
;	---------------------------------
whPieces
	;   $36,$37,$27,$33,$35,$30
	;    54, 55, 39, 51, 53, 48
	.db whQ,whR,whB,whN,whP,whK
;
;	------------------------------------------------------------------
blkRook
	.db blR,blN,blB,blK,blQ,blB,blN,blR
;
;	------------------------------------------------------------------
;
; PIECE: this sets up pointers to possible move tables and number of steps and
; directions.
;
PIECE
	ld e,l			; save the current position of the piece

	xor a			; clear the list 
	ld (newList),a		; of moves

	ld a,(hl)		; content of the current position 
	ld d,a			; store it

	call PSC		; is a piece? Q(5),R(4),B(3),N(2),P(1),K(0)
	ret nz			; no, return

	ld bc,$0801		; max. nr. of directions and moves
	ld hl,theKing		; pointer to the table of moves
	and a			; the "K"ing?
	jr z,MOVE		; creating list of moves

	ld l,blkPawn & 255	; pointer to the table of moves
	dec a			; is it a "P"awn?
	jr z,PAWN		; creating list of moves

	ld l,akNight & 255	; pointer to the table of moves
	dec a			; is it a k"N"ight?
	jr z,MOVE		; creating list of moves

	ld bc,$0408		; max. nr. of directions and moves
	ld l,aBishop & 255	; pointer to the table of moves
	dec a			; is it a "B"ishop?
	jr z,MOVE		; creating list of moves

	ld l,aRook & 255	; pointer to the table of moves
	dec a			; is it a "R"ook?
	jr z,MOVE		; creating list of moves

	ld b,c			; 8 directions and 8 moves
        			; the "Q"ueen
;
;	------------------------------------------------------------------
;
; MOVE: produces a list of all legal moves available to the piece under
; consideration.
;
MOVE
	ld a,e			; the current position of the piece
MOVE1
	add a,(hl)		; new position based on the table of moves
	push af			; save position
	push hl			; save table of movements' pointer
	push bc			; save max. number of moves and directions

	call STR		; analyse content

	cp $02			; if it's not empty (1) or not an opponent (0)
	jr nc,MOVE2		; then see the next possible move

	push af			; otherwise save position property
	call ALIST		; and add to the list of moves

	pop af			; set back the position property
	and a			; if it's an opponent, then we can't go in that direction
	jr z,MOVE2		; let's see the next possible move

	pop bc			; max. number of moves and the direction
	pop hl			; the pointer of table of moves
	ld a,c			; if this piece (in this direction)
	dec a			; can only move once (one square)
	jr z,MOVE3		; let's see the next possible direction

	pop af			; otherwise set back the address of the position
	jr MOVE1		; then calculate the next move
MOVE2
	pop bc			; max. number of moves and the direction
	pop hl			; the pointer of table of moves
MOVE3
	pop af			; set back the address of the position
	inc hl			; the next item of the table of moves
	djnz MOVE		; if it wasn't the last direction, then repeat from the beginning
	ret			;
;
;	------------------------------------------------------------------
;
; PAWN: produces a list of all possible legal moves including initial double moves.
;
PAWN
        bit 7,d                 ; the pawn's colour
	jr nz,PAWN1		; black?

	ld l,whtPawn & 255	; no - white pawn's table of moves
PAWN1
	ld d,$03		; can move to 3 different directions
PAWN2
	ld a,e			; the pawn's position (lower byte)
PAWN3
	add a,(hl)		; new position based on the table of moves
	push hl			; save pointer of the table of moves
	push af			; save position

	call STR		; analyse content
				; if it's an opponent,
	jr z,PAWN5		; then add to the list of moves

	dec a			; if it's not empty,
	jr nz,PAWN4		; then see the next direction

	ld a,d			; if it's not the
	dec a			; last direction (forward)
	jr nz,PAWN4		; see the next direction

	call ALIST		; otherwise add to the list of moves

	ld a,e			; the pawn's position (lower byte)

	cp wPwnMax & 255	; 2nd row? (white pawn's first move)
	jr c,PAWN6		; if yes, then let's see the next move

	cp bPwnMax & 255	; 7th row? (black pawn's first move)
	jr nc,PAWN6		; if yes, then let's see the next move
PAWN4
	pop af			; set back the address of the position
	pop hl			; the pointer of the table of moves
	dec hl			; set to the next direction
	dec d			; check the direction
	jr nz,PAWN2		; if it wasn't the last, then again
	
	ret     
;
;	------------------------------------------------------------------
PAWN5
	ld a,d			; if the direction is 
	dec a			; not the last (forward)
	call nz,ALIST		; then add to the list of moves

	jr PAWN4		; test the next direction
;
;	------------------------------------------------------------------
PAWN6
	pop af			; set back the address of the position
	pop hl			; the pointer of the table of moves
	ld e,a			; keep the positon's address
	jr PAWN3		; test the new position
;
; ======================================================================== 
; D-FILE
; ==============================================
;
DFile
	.db $76,$76,$76,$76,$76,$76				; rows 1-5

	.db $1D,$09						; 

CountA1	.equ $+7-11+$26
BoardI1	.equ $-1
BoardH1	;;
	.db $E6,$7F,$21,$D1,$40,$01,$06,$00	; row 6
;
;	------------------------------------------------------------------
;
	.db $76,$1E,$09			;;

BoardI2	.equ $-1
BoardH2		
	.db $ED,$B1,$79,$C9,$66,$41,$21,$A9	; row 7
;
;	------------------------------------------------------------------
BoardA2
wPwnMax
	.db $76,$1F,$09			;; 
	.db $43,$34,$7E,$85,$6F,$71,$C9,$06	; row 8
;
;	------------------------------------------------------------------
;
	.db $76,$20,$09			;; 
	.db $08,$2B,$71,$10,$FC,$C9,$00,$00	; row 9
;
;	------------------------------------------------------------------
;
	.db $76,$21,$09			;; 
	.db $00,$00,$00,$00,$00,$00,$00,$00	; row 10
;
;	------------------------------------------------------------------
;
	.db $76,$22,$09	;; 
	.db $21,$D7,$40,$01,$08,$08,$CD,$F6	; row 11
;
;	------------------------------------------------------------------
bPwnMax
	.db $76,$23,$09			;; 
	.db $42,$ED,$A0,$10,$F9,$C9,$21,$A9	; row 12
;
;	------------------------------------------------------------------
BoardA7
	.db $76,$24,$09					
BoardH8					;; 
	.db $43,$35,$7E,$3C,$C8,$85,$6F,$7E	; row 13
;
;	------------------------------------------------------------------
BrdOut
	.db $76		

BrdSize	.equ $-BoardH1

	.db $09,$09
BrdOut3
	.db $2D,$2C,$2B,$2A,$29,$28,$27,$26,$76,$76		; rows 14-15
;
;	------------------------------------------------------------------
;
INPUT3	.equ $+3
INPUT			;; 
	.db $12,$12,$12,$12,$12

	.db $76							; 

	.db $76,$76,$76,$76,$76,$76,$76,$76			; rows 17-24
;
;	================================================================== 
;
origPos .equ $+1
PMOVE
	ld hl,BoardH1		; original position
	ld a,(de)		; the content of the new position
	ld c,a			; to the register "C"
	ld a,(hl)		; the content of the original position
	ld (hl),$00		; (deleting temporarily)
	ld (de),a		; to the new position
	ld b,a			; and to register "B"

	exx			; save the main register set
	and a			; CY=0
	call SHIFT		; save the list of moves
;
;	------------------------------------------------------------------
;
; CHK locates current mover's Kings and stores the position in the attack
; register.
;
CHK
	ld a,(MoverCol)	; current mover's colour: $80(black), $00(white)
	add a,whK		; code for the "K"ing
	ld hl,BoardH1		; the board's starting address
	ld b,a			; setting the counter (bc) for
	cpir			; search

	ld a,l			; 
	dec a			; the result
;
;	------------------------------------------------------------------
;
; SQuare ATtack: determines whether the opposition can attack the square in the
; attack register.
;
SQ_AT
	ld (RAttack),a		; save the position
	ld b,BrdSize & 255	; check upon the number of screenposition (max. 86)
	ld hl,BoardH1-1	; the board's starting address (-1)
SQ_AT1
	inc hl			; the next position
	push hl			; save it
	push bc			; save the counter
	call STR2		; analyse the content

	jr nz,SQ_AT3		; if not opponent, then next

        ex de,hl                ; (save HL)
	call CHGMV		; changing the colour of the mover for the check up
        ex de,hl                ; (restore HL)

	call PIECE		; creating the list of moves

	call CHGMV		; set back the colour of the mover
SQ_AT2
	call TL			; test the list
	jr z,SQ_AT3		; if empty, see the next position

RAttack	.equ $+1
				; is the address in the attack register
	cp BoardH1 & 255	; on the list?
	jr nz,SQ_AT2		; if not, see the next component on the list

	pop bc			; set back the counter
	pop hl			; in HL the address of the attacker
	scf			; CY=1 indicates that there's an attack
	ret			;
SQ_AT3
	pop bc			; set back the counter
	pop hl			; the screen position which was checked upon
	djnz SQ_AT1		; if the last position is safe as well

	and a			; CY=0 indicates that there's no attack
	ret			;
;
;	------------------------------------------------------------------
;
; INC determines whether a square is being attacked.
;
INC_
	ld a,l			; the lower byte of the address

	exx			; save the main register set

	call SQ_AT		; is this position under attack?
				; (CY=1, if yes)
	exx			; set back the main register set

	ld a,d			;
	ret			;
;
;	------------------------------------------------------------------
;
; SCORE provides a move score based on the following:
; first, the "To" position results in taking of a piece.
; Second, the "From" position is attacked.
; Third, the "To" position is attacked.
; Fourth, "To" enables the computer to obtain a check
; and finally the "From" position is defended.
;
; The current move score is then compared with the previous best and if this is
; superior, the move is saved as the best so far.
;
SCORE
	push hl			; save the original position's address
	push bc			; save the content of the positions (original/new)
	push de			; save the new position's address

	push hl			; save the original position's address
	push bc			; save the content of the positions (original/new)

        ld h,b                  ; store the content of the original position and the
        ld (oldScore+3),hl      ; address of the original position (lower byte), and
        ld (oldScore+1),de      ; store the new position's address, as score parameters

	call PSC		; value of the piece: Q(5), R(4), B(3), N(2), P(1)

	add a,h			; + $40 (64; whPieces upper byte)
	ld d,a			; saved in D

	pop af			; A= piece in the original position

	call PSC		; value of the piece: Q(5), R(4), B(3), N(2), P(1)

	pop hl			; set back the original position
	call INC_		; is this position under attack? (CY=1, if it is)

	jr nc,SCORE1		;

	add a,c			; if it is, then change the value of the move
SCORE1
	pop hl			; reading the address of the new position and
	pop de			; the original(D)/new(E) content
				; content of the new position
	ld (hl),d		; changed to the content of the original position
	push hl			; save the address of the new position
	push de			; save the content of the positions (original/new)

	ld d,a			; and save it to a temporary repertory

	call INC_		; is the new position under attack? (CY=1, if it is)

	jr nc,SCORE2		;

	sub c			; if yes, then set back the previous value of the move
SCORE2
	push af			; save the calculated value of the move

	call CHGMV		; changing the colour of the mover

	call CHK		; check?

	pop bc			; (calculated value to register "B")
	jr nc,SCORE3		;

	inc b			; if yes, then increase the value of the move
	inc b			; by 2
SCORE3
	pop de			; reading the original(D)/new(E) content
	pop hl			; set back the content of
	ld (hl),e		; the new position

	pop hl			; change the colour of the 
	call CHG		; original position

	call INC_		; is this position under attack? (CY=1, if it is)

	jr nc,SCORE4		;
	dec b			; if it is, then lower the value of the move
SCORE4
	call CHG		; set back the colour of the position

	call CHGMV		; set back the colour of the mover

	ld a,b			; save the calculated value of the
	ld de,oldScore		;
	ld (de),a		; move
	ld hl,newScore		; if the previously calculated
	cp (hl)			; value is smaller,
	ret c			;

	ld bc,$0005		; then copy the data of the new position
	jr SHIFT1		; as a replacement for the old data
;
;	------------------------------------------------------------------
;
; Shift moves the current move list to a safe position whilst Check is being
; evaluated, and then recovers the move list on completion. It is also used to
; shift the best move so far up into the move list.
;
SHIFT
	ld hl,oldList		; address of the saved list
	ld de,newList		; address of the list of moves
	ld bc,$001c		; max. 28 bytes
	jr c,SHIFT2		; if CY=1, then set back
SHIFT1
	ex de,hl		; if CY=0, then save
SHIFT2
	ldir			; moving the data
	ret			;
;
;	------------------------------------------------------------------
;
; MPSCAN: scans the board for computer pieces and, using move and score,
; determines all legal moves and saves the best.
;
MPSCAN
	xor a			; initialise (to zero) the
	ld (newScore),a		; calculated value of the move

	ld b,BrdSize & 255	; we look through max. 86 positions
	ld hl,BoardH1-1		; starting with the board's first component
MPSCAN1
	inc hl			; pointer to the next position on the board
	push hl			; save the position on the board
	push bc			; save the counter
	call STR2		; analyse content

	cp $03			; is it our own piece?
	jr nz,MPSCAN5		; if not, then check the next position

	ld (origPos),hl		; save the position
	call PIECE		; creating list of moves
MPSCAN2
	call TL			; a component of the list
	jr z,MPSCAN5		; if empty, then check the next position

	ld e,a			; a possible new position for the piece
	ld d,BoardH1/256	; 
	call PMOVE		; move the piece
				; save the list of moves
				; check? (is the king under attack?)
	exx			; set back the main register set

	ld (hl),b		; set back the content of the original position
	ld a,c			; and
	ld (de),a		; the content of the new position
				; if the king isn't under attack
	call nc,SCORE		; choose the best possible move

	scf			; CY=1
	call SHIFT		; set back of the list of moves
	jr MPSCAN2		; the next component of the list of moves
MPSCAN5
	pop bc			; the current value of the counter and
	pop hl			; the position on the board
	djnz MPSCAN1		; next position

	ld a,(newScore)		; this was the last position
	and a			; checking the result

	jr z,GameOver		; if the value of the move is zero, then the game ends

	ld de,(newScore+3)	; original content(D) / position(E)
	ld hl,(newScore+1)	; the address of the new position
        ld (hl),d               ; original content to the new position (move)
;
;	------------------------------------------------------------------
ChkProm
	ld a,$7f		; mask out the color bit
	and (hl)		; of the piece
	cp whP			; is it a pawn?
	jr nz,ChkProm2		; if no, then no promotion
	
	ld a,l			; is it 
	cp BoardH2 & 255	; in row 1?
	jr c,ChkProm1		; if yes, jump to promotion

	cp BoardH8 &255	; is it in row 8?
	jr c,ChkProm2		; if no, then no promotion
ChkProm1
	ld a,whQ-whP		; offset to Queen
	add a,(hl)		; change: Pawn -> Queen
	ld (hl),a		; store the Queen
ChkProm2
        ld l,e                  ; the address of the original position
;
;	------------------------------------------------------------------
CHGSQ
	ld a,$01		; the even-numbered
	and l			; squares are white
	rrca			; the odd ones
	ld (hl),a		; are black
;
;	------------------------------------------------------------------
CHGMV
	ld hl,MoverCol		; the colour of the mover
;
;	------------------------------------------------------------------
CHG
	ld a,(hl)		; changing the
	add a,$80		; colour of a piece
	ld (hl),a		; or position
	ret			;
;
;	==================================================================
GameOver
	pop bc			; clear the stack
	call CHGMV		; change the colour of the mover

	ld hl,INPUT3		; set cursor position
	ld bc,$022E		; possible character codes ("I".."J")
	call KYBD0		; get and check the "from" position
;
;	------------------------------------------------------------------
Restart
	ld e,l			; save a board(out) address (lower byte)
;
;	------------------------------------------------------------------
ClrBrd
	ld l,BrdOut3 & 255	; set pointer
	ld c,8			; 8 rows
ClrBrd1
	dec hl			; skip "chess board" character
	dec hl			; skip row number
	dec hl			; skip "N/L" character
	ld b,8			; 8 columns
ClrBrd2
	dec hl			; next position

	ld a,$01		; the even-numbered
	and l			; squares are white
	rrca			; the odd ones
	ld (hl),a		; are black

	djnz ClrBrd2		; if it is not the last, then again

	dec c			; next row
	jr nz,ClrBrd1		; if it is not the last, then back
;
;	------------------------------------------------------------------
;
	ld a,BoardI2 & 255	; test the board(out) address
	cp e			; points to "I2"?
	jr nz,Opening		; if yes, then opening
;
;	------------------------------------------------------------------
brdSETUP
	ld bc,$051b		; the INPUT line consists of 5 "." character
	call GetPos		; get the "from" position

	jr z,DRIVER		; if it is "I1", then let's play

	exx			; save the board position's address
	ld hl,INPUT3		; next input position
brdSUP1
	ld bc,$1127		; possible character codes ("B".."R")
	call TKP		; reading the keyboard

	push hl			; save the piece code position
	call PSC		; is a piece? Q(5),R(4),B(3),N(2),P(1),K(0)
	pop hl			; restore the piece code position

	jr nz,brdSUP1		; no, ask again

	push hl			; save the piece code position
	inc hl			;
	ld bc,$021D		; possible character codes ("1".."2")
	call TKP		; reading the keyboard

	and $01			; number from character ("1" -> 1; "2" -> 0)
	rrca			; $01 -> $80 (black); $00 -> $00 (white)
	pop hl			; restore the piece code position
	add a,(hl)		; set piece color

	exx			; get the board position's address
	ld (hl),a		; place the piece

	jr brdSETUP		; next piece
;
;	------------------------------------------------------------------
Opening
	ex de,hl		; ld de,BoardH1	
	call FillBrd		; place all the pieces in row 1

	ld e,BoardH8 & 255	;
	call FillBrd		; place all the pieces in row 8

	ld h,d			;
	ld l,BoardA7 & 255	;
	ld c,blP		;
	call FillPawn		; place all the black pawns

	ld l,BoardA2 & 255	;
	ld c,whP		;
	call FillPawn		; place all the white pawns
;
;	------------------------------------------------------------------
;
#ifdef QPawn	;

OpenPos	.equ BoardH2+4		; "Queen's Pawn Game"

#else		;

OpenPos	.equ BoardH2+3		; "King's Pawn Game"

#endif
;
;	---------------------------------
;
	ld e,OpenPos & 255	; pointer to the "from" position
	ld hl,OpenPos+22	; pointer to the "to" position
	ld b,whP		; "white" moves
	ld c,(hl)		; get the position's color
;
;	==================================================================
DRIVER4
	ld (hl),b		; move the piece to the original position 
	ld a,c			; set back the original value
	ld (de),a		; of the new position
;
; ************************************************************************
;
; DRIVER:
; Main control logic, uses all the other subroutines to provide program control.
;
DRIVER
	ld bc,$0512		; the INPUT line consists of 5 "-" character
	call GetPos		; get the "from" position
				; out of board (I1..I9)?
	jr z,Restart		; if yes, then restart the game
				; is it our own piece?
	jr c,DRIVER		; if not, then from the beginning

	ld (origPos),hl		; save the "from" position
	call PIECE		; creating the list of moves

	ld hl,INPUT3		; 
	call GetPos2		; get and check the target position
				; if not empty, or not one of the pieces
	jr nc,DRIVER		; of the opponent, then from the beginning
	ex de,hl		; otherwise save the new position
DRIVER2
	call TL			; testing the list
	jr z,DRIVER		; if empty, then from the beginning

	cp c			; is it on the list?
	jr nz,DRIVER2		; if not, see the next item of the list

	call PMOVE		; move piece to the new position

	exx			; set back the main register set
	jr c,DRIVER4		; cancel the move, if the king is under attack

	ex de,hl		; checking possibility of the promotion, then
	call ChkProm		; setting of the colours of the old position and
				; changing the colour of the mover
	call MPSCAN		; ZEDDY moves

	jr DRIVER		; USER moves
;
;	------------------------------------------------------------------
;
oldScore			;
newScore .equ $+5		;

newList .equ $+10		;
oldList .equ $+38		;
;
;	==================================================================
START
	ld hl,BoardH1-3		; the address of the first 8 bytes
	ld de,PSC		; and the destination ($4000)
START1
	ld c,$03		; skip the N/L and 2 other
	add hl,bc		; characters in DFile
	;
	ld c,$08		; then copy 8 bytes
	ldir
	;
	dec a			; was the row 8?
	jr nz,START1		; no, repeat
	;
	ld sp,$4400             ; SP points the top of RAM (1K)
	ld e,BoardI1 & 255	; simulate a restart
	jp ClrBrd               ; and (re)start
	;
	; ---------------------------------------------------------------------
	;
	.db $76			; Line 2 end
;
;	=======================================================================
eof_Prg
	.db $80
eof_Vars
.end